@@ -12,32 +12,35 @@ OC.L10N.register(
     "Connect" : "Kopla til",
     "Signature" : "Signatur",
     "Cancel" : "Avbryt",
-    "Retry" : "Forsøk att",
+    "Delete" : "Ta bort",
+    "Search" : "Search",
     "Send" : "Send",
+    "Refresh" : "Last inn på nytt",
+    "Choose" : "Vel",
     "Favorite" : "Favoritt",
     "Select" : "Vel",
-    "Move" : "Move",
+    "Back" : "Tilbake",
+    "Description" : "Skildring",
     "Create" : "Lag",
     "Comment" : "Kommentér",
-    "Choose" : "Vel",
     "Other" : "Anna",
     "Reply" : "Svare",
+    "Move" : "Move",
     "Quota" : "Kvote",
     "Saving" : "Lagrar",
-    "Delete" : "Ta bort",
+    "Retry" : "Forsøk att",
     "Add" : "Legg til",
-    "Tags" : "Emneord",
-    "Search" : "Search",
+    "Close" : "Lat att",
     "Date" : "Dato",
+    "Tags" : "Emneord",
     "Disabled" : "Deaktivert",
-    "Set" : "Sett",
     "Remove" : "Fjern",
+    "Client ID" : "Klient-ID",
     "General" : "Generelt",
     "User" : "Bruker",
     "Host" : "Tenar",
     "Port" : "Port",
-    "Keyboard shortcuts" : "Tastatursnarvegar",
-    "Refresh" : "Last inn på nytt",
+    "Certificate" : "Sertifikat",
     "All" : "Alle",
     "Favorites" : "Favorit",
     "Sent" : "Sendt",
